
local M = {}



local function angleBetween( srcX, srcY, dstX, dstY )
	local angle = ( math.deg( math.atan2( dstY-srcY, dstX-srcX ) )+90 )
	return angle % 360
end

local function distBetween( x1, y1, x2, y2 )
	local xFactor = x2 - x1
	local yFactor = y2 - y1
	local dist = math.sqrt( (xFactor*xFactor) + (yFactor*yFactor) )
	return dist
end



local function follow( params, obj, pathPoints, pathPrecision )

	local function nextTransition()

		if ( obj.nextPoint > #pathPoints ) then
			--print( "FINISHED" )

		else
			--set variable for time of transition on this segment
			local transTime = params.segmentTime
			--if "params.constantRate" is true, adjust time according to segment distance
			if ( params.constantRate == true ) then
				local dist = distBetween( obj.x, obj.y, pathPoints[obj.nextPoint].x, pathPoints[obj.nextPoint].y )
				transTime = (dist/pathPrecision) * params.segmentTime
			end
			
			--rotate object to face next point
			if ( obj.nextPoint < #pathPoints ) then
				obj.rotation = angleBetween( obj.x, obj.y, pathPoints[obj.nextPoint].x, pathPoints[obj.nextPoint].y )
			end
			
			--transition along segment
			transition.to( obj, {
				tag = "moveObject",
				time = transTime,
				x = pathPoints[obj.nextPoint].x,
				y = pathPoints[obj.nextPoint].y,
				onComplete = nextTransition
			})
			obj.nextPoint = obj.nextPoint+1

		end
	end
	
	obj.nextPoint = 2
	nextTransition()

end



function M.init( params, pathPoints, pathPrecision, startPoint )

	local follower = display.newPolygon( 0, 0, { 0,-28, 30,28, 0,20, -30,28 } )
	follower:setFillColor( 1 )
	follower.x = startPoint.x
	follower.y = startPoint.y

	follower.rotation = angleBetween( pathPoints[1].x, pathPoints[1].y, pathPoints[2].x, pathPoints[2].y )

	--add follower to module for reference
	M.obj = follower
	
	local precision = pathPrecision
	if ( pathPrecision == 0 ) then
		precision = distBetween( pathPoints[1].x, pathPoints[1].y, pathPoints[2].x, pathPoints[2].y )
	end
	
	--if "showPoints" is true, plot points along path
	if ( params.showPoints == true ) then
		local pathPointsGroup = display.newGroup() ; pathPointsGroup:toBack()
		for p = 1,#pathPoints do
			local dot = display.newCircle( pathPointsGroup, 0, 0, 8 )
			dot:setFillColor( 1, 1, 1, 0.4 )
			dot.x = pathPoints[p].x
			dot.y = pathPoints[p].y
		end
		M.ppg = pathPointsGroup
	end

	follow( params, follower, pathPoints, precision )
	
end


return M
