
-- Abstract: Curve Drawing
--
-- Version: 1.0
--
-- Sample code is MIT licensed, see http://www.coronalabs.com/links/code/license
-- Copyright (C) 2014 Corona Labs Inc. All Rights Reserved.
------------------------------------------------------------

display.setStatusBar( display.HiddenStatusBar )

--require "follow.lua" module (remove if using only main draw code)
local followModule = require( "follow" )
--declare follow module parameters (remove if using only main draw code)
local followParams = { segmentTime=50, constantRate=true, showPoints=true }

local curve
local pathPoints = {}
local anchorCount = 0
local anchorPoints = { {},{},{},{} }
local curveHandleLines = {}

--adjust this number to effect the "smoothness" of the curve (total number of segments)
local curveSegments = 100



local function renderHandles( endX, endY, startAnchor, endAnchor, handle )

	--move handle anchor in unison with touch
	endAnchor.x = endX
	endAnchor.y = endY

	local thisHandle
	if ( handle == "first" ) then
		--if a handle line already exists, remove it so it can be replaced
		if ( curveHandleLines[1] ) then display.remove( curveHandleLines[1] ) end
		--render handle line
		curveHandleLines[1] = display.newLine( startAnchor.x, startAnchor.y, endAnchor.x, endAnchor.y )
		thisHandle = curveHandleLines[1]
	elseif ( handle == "second" ) then
		--if a handle line already exists, remove it so it can be replaced
		if ( curveHandleLines[2] ) then display.remove( curveHandleLines[2] ) end
		--render handle line
		curveHandleLines[2] = display.newLine( startAnchor.x, startAnchor.y, endAnchor.x, endAnchor.y )
		thisHandle = curveHandleLines[2]
	end
	thisHandle:setStrokeColor( 1, 1, 1, 0.5 )
	thisHandle.strokeWidth = 4
	thisHandle:toBack()

end



local function renderCurve( segNum, r, g, b )

	local inc = ( 1.0 / segNum )

	for i = 1,#anchorPoints,4 do

		local t = 0
		local t1 = 0
		local i = 1

		for j = 1,segNum do 

			t1 = 1.0 - t
			local t1_3 = t1 * t1 * t1
			local t1_3a = (3*t) * (t1*t1)
			local t1_3b = (3*(t*t)) * t1
			local t1_3c = t * t * t
			local p1 = anchorPoints[i]
			local p2 = anchorPoints[i+1]
			local p3 = anchorPoints[i+2]
			local p4 = anchorPoints[i+3]
	
			local x = t1_3 * p1.x
			x = x + t1_3a * p2.x
			x = x + t1_3b * p3.x
			x = x + t1_3c * p4.x

			local y = t1_3 * p1.y
			y = y + t1_3a * p2.y
			y = y + t1_3b * p3.y
			y = y + t1_3c * p4.y

			pathPoints[j].x = x
			pathPoints[j].y = y
			t = t + inc
		end
	end
	
	--reset/clear core
	if curve then display.remove( curve ) end

	curve = display.newLine( pathPoints[1].x, pathPoints[1].y, pathPoints[2].x, pathPoints[2].y )

	for i = 3,#pathPoints do
		curve:append( pathPoints[i].x, pathPoints[i].y )
	end
	curve:append( anchorPoints[4].x, anchorPoints[4].y )
	curve:setStrokeColor( r, g, b )
	curve.strokeWidth = 4
	curve:toBack()

end



local function dragHandles( event )

	local t = event.target

	if ( event.phase == "began" ) then
	
	--reset/clear follow module items (remove these lines if not using "follow.lua")
			transition.cancel( "moveObject" )
			if ( followModule.obj ) then display.remove( followModule.obj ) ; followModule.obj = nil end
			if ( followModule.ppg ) then
				for p = followModule.ppg.numChildren,1,-1 do display.remove( followModule.ppg[p] ) end
			end
			
		display.currentStage:setFocus( t )
		t.isFocus = true

	elseif ( t.isFocus ) then

		if ( event.phase == "moved" ) then

			renderCurve( curveSegments, 1, 1, 1 )
			
			if ( t == anchorPoints[1] ) then
				renderHandles( event.x, event.y, anchorPoints[2], anchorPoints[1], "first" )
			elseif ( t == anchorPoints[2] ) then
				renderHandles( event.x, event.y, anchorPoints[1], anchorPoints[2], "first" )
			elseif ( t == anchorPoints[3] ) then
				renderHandles( event.x, event.y, anchorPoints[4], anchorPoints[3], "second" )
			elseif ( t == anchorPoints[4] ) then
				renderHandles( event.x, event.y, anchorPoints[3], anchorPoints[4], "second" )
			end

		elseif ( event.phase == "ended" or event.phase == "cancelled" ) then

			display.currentStage:setFocus( nil )
			t.isFocus = false
			renderCurve( curveSegments, 1, 0.5, 0 )

			--start follow module
			if ( #pathPoints > 1 ) then
				followModule.init( followParams, pathPoints, 0, anchorPoints[1] )
			end

		end
	end

	return true
end



local function addCurveAnchors( event )

	--if both points and both control points have not yet been placed, proceed with routine
	if ( anchorCount <= 4 ) then
	
		if ( event.phase == "began" ) then
				
			anchorCount = anchorCount+1
			local point = display.newCircle( event.x, event.y, 20 )
			if ( anchorCount == 1 ) then point:setFillColor( 0.2, 0.8, 0.4 ) else point:setFillColor( 1, 0, 0.2 ) end

			if ( anchorCount == 1 ) then
				anchorPoints[1] = point  --insert start point at first index
			elseif ( anchorCount == 3 ) then
				anchorPoints[4] = point  --insert end point at last index
			end

			anchorCount = anchorCount+1
			local anc = display.newCircle( event.x, event.y, 20 )
			anc:setFillColor( 0.4 )
			anc:toBack()
			
			if ( anchorCount == 2 ) then
				anchorPoints[2] = anc  --insert first control point at second index
			elseif ( anchorCount == 4 ) then
				anchorPoints[3] = anc  --insert second control point at third index
				--render curve
				renderCurve( curveSegments, 1, 0.5, 0 )
			end

		elseif ( event.phase == "moved" ) then

			--on touch move, render/re-render handles and curve line if applicable
			if ( anchorCount == 2 ) then
				renderHandles( event.x, event.y, anchorPoints[1], anchorPoints[2], "first" )
			elseif ( anchorCount == 4 ) then
				renderHandles( event.x, event.y, anchorPoints[4], anchorPoints[3], "second" )
				--(re)render curve
				renderCurve( curveSegments, 1, 1, 1 )
			end

		elseif ( event.phase == "ended" or event.phase == "cancelled" ) then

			if ( anchorCount == 4 ) then
				display.currentStage:removeEventListener( "touch", addCurveAnchors )
				
				--reset/clear follow module items (remove these lines if not using "follow.lua")
				transition.cancel( "moveObject" )
				if ( followModule.obj ) then display.remove( followModule.obj ) ; followModule.obj = nil end
				if ( followModule.ppg ) then
					for p = followModule.ppg.numChildren,1,-1 do display.remove( followModule.ppg[p] ) end
				end
		
				--add touch listener to each handle
				for i = 1,#anchorPoints do 
					anchorPoints[i]:addEventListener( "touch", dragHandles )
				end
				--(re)render curve
				renderCurve( curveSegments, 1, 0.5, 0 )
				
				--start follow module
				if ( #pathPoints > 1 ) then
					followModule.init( followParams, pathPoints, 0, anchorPoints[1] )
				end

			end

		end
	end

	return true
end



for j = 1,curveSegments do
	pathPoints[#pathPoints+1] = { x=0, y=0 }
end

display.currentStage:addEventListener( "touch", addCurveAnchors )
